@@ -573,13 +573,23 @@ class MailingList(object):
     def get_mail_recipients(self, message):
         assert isinstance(message, AgentMessage)
         recipients = set()
-        if message.receiver is not None:
-            recipients.update(self.directory[message.receiver][None])
-            if message.topic in self.directory[message.receiver]:
+
+        if message.receiver is None:  # it's a broadcast: Go to topic.
+            pass
+        else:  # it's a direct message.
+            if None in self.directory[message.receiver]:  # the receiver exists as an agent.
+                # retrieve set of subscribers of messages send to this agent no matter the topic.
+                recipients.update(self.directory[message.receiver][None])
+
+            if message.topic in self.directory[message.receiver]:  # there are subscribers who
+                # are interested only in the agent when it receives a message with a specific topic.
                 recipients.update(self.directory[message.receiver][message.topic])
 
-        if message.topic in self.directory:
-            recipients.update(self.directory[message.topic][None])
+        # Topic:
+        if message.topic in self.directory:  # there are subscribers interested in this topic no
+            # matter which agent is supposed to receive the message
+            if None in self.directory[message.topic]:
+                recipients.update(self.directory[message.topic][None])
 
         return recipients
 
